==> billiards-0.1.tar.gz: billiards-0.1/src/parametric.c <==
/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <lua.h>
#include <lauxlib.h>
#include <GL/gl.h>
#include <GL/glext.h>
#include "texture.h"
#include "parametric.h"
#include "light.h"

static const GLchar *header =
"uniform sampler2D parameterSampler;			   \n"
"uniform vec3 parameterConstant;			   \n"
"							   \n"
"vec3 parameterSum(vec2 texcoord)			   \n"
"{							   \n"    
"    return parameterConstant + 			   \n"
"           vec3(texture2D(parameterSampler, texcoord));   \n"
"}							   \n"    
"							   \n"
"#line 1						   \n";

@implementation Parametric

-(Parametric *)init
{
    self->parametric.texture = nil;
    self->parametric.values[0] = 0;
    self->parametric.values[1] = 0;
    self->parametric.values[2] = 0;

    self->parametric.constant = glGetUniformLocationARB (self->program,
						   "parameterConstant");
    self->parametric.sampler = glGetUniformLocationARB (self->program,
						  "parameterSampler");
    
    [super init];

    return self;
}

-(void)attachVertexSource: (const GLchar *)source
{
    GLchar *newsource;

    newsource = alloca(strlen(header) + strlen(source) + 2);

    strcpy (newsource, header);
    strcat (newsource, source);

    [super attachVertexSource: (const GLchar *)newsource];
}

-(void)attachFragmentSource: (const GLchar *)source
{
    GLchar *newsource;

    newsource = alloca(strlen(header) + strlen(source) + 2);

    strcpy (newsource, header);
    strcat (newsource, source);
    
    [super attachFragmentSource: (const GLchar *)newsource];
}

-(void) get: (lua_State *)L
{
    const char *k;

    k = lua_tostring(L, 2);

    if (!strcmp(k, "parameter")) {
	lua_getmetatable (L, 1);
	lua_replace (L, 1);
	lua_gettable (L, 1);
    } else {
	[super get: L];
    }
}

-(void) set: (lua_State *)L
{    
    const char *k;
    int i;

    k = lua_tostring(L, 2);

    if (!strcmp(k, "parameter")) {
        if(lua_istable(L, 3)) {
            for(i = 0 ; i < 3 ; i += 1) {
                lua_rawgeti(L, 3, i + 1);
                self->parametric.values[i] = lua_tonumber(L, -1);
                
                lua_pop(L, 1);
            }
	    
	    self->parametric.texture = nil;
        } else if(lua_isnumber(L, 3)) {
	    self->parametric.values[0] = lua_tonumber(L, 3);
	    
            for(i = 1 ; i < 3 ; i += 1) {
                self->parametric.values[i] = 0;
            }
	    
	    self->parametric.texture = nil;
        } else if(lua_isuserdata (L, 3)) {
            for(i = 0 ; i < 3 ; i += 1) {
                self->parametric.values[i] = 0;
            }
	    
	    self->parametric.texture = *(id *)lua_touserdata (L, 3);
	}

	lua_getmetatable (L, 1);
	lua_replace (L, 1);
	lua_settable (L, 1);
    } else {
	[super set: L];
    }
}

-(void)traverse: (lua_State *)L
{
    GLint unit;
    
    glGetIntegerv(GL_ACTIVE_TEXTURE, &unit);
    glActiveTexture (unit + 1);

    if(self->parametric.texture) {
	glBindTexture(GL_TEXTURE_2D, [self->parametric.texture index]);
    } else {
	glBindTexture(GL_TEXTURE_2D, 0);
    }

    glUniform3fvARB (self->parametric.constant, 1, self->parametric.values);
    glUniform1iARB (self->parametric.sampler, unit + 1 - GL_TEXTURE0);
    
    [super traverse: L];
}

@end

==> billiards-0.2.2.tar.gz: billiards-0.2.2/src/parametric.c <==
/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <lua.h>
#include <lauxlib.h>
#include <GL/gl.h>
#include <GL/glext.h>
#include "texture.h"
#include "parametric.h"
#include "light.h"

static const GLchar *header =
"uniform sampler2D parameterSampler;			   \n"
"uniform vec3 parameterConstant;			   \n"
"							   \n"
"vec3 parameterSum(vec2 texcoord)			   \n"
"{							   \n"    
"    return parameterConstant + 			   \n"
"           vec3(texture2D(parameterSampler, texcoord));   \n"
"}							   \n"    
"							   \n"
"#line 1						   \n";

@implementation Parametric

-(Parametric *)init
{
    self->parametric.texture = nil;
    self->parametric.values[0] = 0;
    self->parametric.values[1] = 0;
    self->parametric.values[2] = 0;

    self->parametric.constant = glGetUniformLocationARB (self->program,
						   "parameterConstant");
    self->parametric.sampler = glGetUniformLocationARB (self->program,
						  "parameterSampler");
    
    [super init];

    return self;
}

-(void)attachVertexSource: (const GLchar *)source
{
    GLchar *newsource;

    newsource = alloca(strlen(header) + strlen(source) + 2);

    strcpy (newsource, header);
    strcat (newsource, source);

    [super attachVertexSource: (const GLchar *)newsource];
}

-(void)attachFragmentSource: (const GLchar *)source
{
    GLchar *newsource;

    newsource = alloca(strlen(header) + strlen(source) + 2);

    strcpy (newsource, header);
    strcat (newsource, source);
    
    [super attachFragmentSource: (const GLchar *)newsource];
}

-(void) get
{
    const char *k;

    k = lua_tostring (_L, 2);

    if (!xstrcmp(k, "parameter")) {
	lua_getmetatable (_L, 1);
	lua_replace (_L, 1);
	lua_gettable (_L, 1);
    } else {
	[super get];
    }
}

-(void) set
{    
    const char *k;
    int i;

    k = lua_tostring (_L, 2);

    if (!xstrcmp(k, "parameter")) {
        if(lua_istable (_L, 3)) {
            for(i = 0 ; i < 3 ; i += 1) {
                lua_rawgeti (_L, 3, i + 1);
                self->parametric.values[i] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);
            }
	    
	    self->parametric.texture = nil;
        } else if(lua_isnumber (_L, 3)) {
	    self->parametric.values[0] = lua_tonumber (_L, 3);
	    
            for(i = 1 ; i < 3 ; i += 1) {
                self->parametric.values[i] = 0;
            }
	    
	    self->parametric.texture = nil;
        } else if(lua_isuserdata (_L, 3)) {
            for(i = 0 ; i < 3 ; i += 1) {
                self->parametric.values[i] = 0;
            }
	    
	    self->parametric.texture = *(id *)lua_touserdata (_L, 3);
	}

	lua_getmetatable (_L, 1);
	lua_replace (_L, 1);
	lua_settable (_L, 1);
    } else {
	[super set];
    }
}

-(void)traverse
{
    GLint unit;
    
    glGetIntegerv(GL_ACTIVE_TEXTURE, &unit);

    if(self->parametric.texture) {
	glBindTexture(GL_TEXTURE_2D, [self->parametric.texture index]);
    } else {
	glBindTexture(GL_TEXTURE_2D, 0);
    }

    glUniform3fvARB (self->parametric.constant, 1, self->parametric.values);
    glUniform1iARB (self->parametric.sampler, unit - GL_TEXTURE0);

    glActiveTexture (unit + 1);

    [super traverse];
}

@end
